import 'dart:math';

import 'package:flutter/cupertino.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/gestures.dart';
import 'package:flutter_webview_desktop/src/desktop_webview_widget.dart';
import 'package:webview_flutter_platform_interface/webview_flutter_platform_interface.dart';

class DesktopWebviewPlatform implements WebViewPlatform {
  @override
  Widget build({
    required BuildContext context,
    required CreationParams creationParams,
    required WebViewPlatformCallbacksHandler webViewPlatformCallbacksHandler,
    required JavascriptChannelRegistry javascriptChannelRegistry,
    WebViewPlatformCreatedCallback? onWebViewPlatformCreated,
    Set<Factory<OneSequenceGestureRecognizer>>? gestureRecognizers,
  }) {
    if (creationParams.initialUrl == null) {
      return Container();
    }
    return LayoutBuilder(
      builder: (context, constraints) {
        return DesktopWebviewWidget(
          width: min(constraints.maxWidth, MediaQuery.of(context).size.width),
          height: min(constraints.maxHeight, MediaQuery.of(context).size.height),
          devicePixelRatio: MediaQuery.of(context).devicePixelRatio,
          initialUrl: creationParams.initialUrl ?? '',
          onFinishedCreated: (controller) {
            if (onWebViewPlatformCreated != null) {
              onWebViewPlatformCreated(controller);
            }
          },
          javascriptChannelRegistry: javascriptChannelRegistry,
          webViewPlatformCallbacksHandler: webViewPlatformCallbacksHandler,
        );
      },
    );
  }

  @override
  Future<bool> clearCookies() {
    if (WebViewCookieManagerPlatform.instance == null) {
      throw Exception('Could not clear cookies as no implementation for WebViewCookieManagerPlatform has been registered.');
    }
    return WebViewCookieManagerPlatform.instance!.clearCookies();
  }
}
