#include "include/flutter_webview_desktop/flutter_webview_desktop_plugin.h"

// This must be included before many other Windows headers.
#include <windows.h>

#include <VersionHelpers.h>

#include <flutter/method_channel.h>
#include <flutter/plugin_registrar_windows.h>
#include <flutter/standard_method_codec.h>
#include <flutter/flutter_view_controller.h>
#include <map>
#include <memory>
#include <sstream>

#include "include/cef_browser.h"
#include "include/wrapper/cef_helpers.h"

#include "webview_bridge.h"
#include "webview_app.h"
namespace
{
  CefRefPtr<WebviewApp> g_app_ = nullptr;
  bool is_init_cef_ = false;
  CefRefPtr<CefCommandLine> CreateCommandLine(const CefMainArgs &main_args)
  {
    CefRefPtr<CefCommandLine> command_line = CefCommandLine::CreateCommandLine();
#if defined(OS_WIN)
    command_line->InitFromString(::GetCommandLineW());
#else
    command_line->InitFromArgv(main_args.argc, main_args.argv);
#endif
    return command_line;
  }
  constexpr auto kMethodInitialize = "initialize";
  constexpr auto kMethodDispose = "dispose";
  constexpr auto kMethodAddJavascriptChannels = "addJavascriptChannels";
  constexpr auto kMethodRemoveJavascriptChannels = "removeJavascriptChannels";

  class FlutterWebviewDesktopPlugin : public flutter::Plugin
  {
  public:
    static void RegisterWithRegistrar(flutter::PluginRegistrarWindows *registrar);

    FlutterWebviewDesktopPlugin(flutter::TextureRegistrar *textures,
                                flutter::BinaryMessenger *messenger);

    virtual ~FlutterWebviewDesktopPlugin();

  private:
    flutter::TextureRegistrar *textures_;
    flutter::BinaryMessenger *messenger_;
    std::unordered_map<int64_t, std::unique_ptr<WebviewBridge>> instances_;

    // WNDCLASS window_class_ = {};
    bool is_init_ = false;
    CefBrowserSettings browser_settings;

    // Information used when creating the native window.
    CefWindowInfo window_info;
    // Called when a method is called on this plugin's channel from Dart.
    void
    HandleMethodCall(
        const flutter::MethodCall<flutter::EncodableValue> &method_call,
        std::unique_ptr<flutter::MethodResult<flutter::EncodableValue>> result);
    void InitWebviewBrowser(std::unique_ptr<flutter::MethodResult<flutter::EncodableValue>> result, int width, int height, const std::string &url, double devicePixelRatio);
    // static LRESULT CALLBACK WndProc(HWND const window,
    //                                 UINT const message,
    //                                 WPARAM const wparam,
    //                                 LPARAM const lparam);
  };

  // static
  void FlutterWebviewDesktopPlugin::RegisterWithRegistrar(
      flutter::PluginRegistrarWindows *registrar)
  {
    auto channel =
        std::make_unique<flutter::MethodChannel<flutter::EncodableValue>>(
            registrar->messenger(), "flutter_webview_desktop",
            &flutter::StandardMethodCodec::GetInstance());

    auto plugin = std::make_unique<FlutterWebviewDesktopPlugin>(registrar->texture_registrar(), registrar->messenger());

    channel->SetMethodCallHandler(
        [plugin_pointer = plugin.get()](const auto &call, auto result)
        {
          plugin_pointer->HandleMethodCall(call, std::move(result));
        });

    registrar->AddPlugin(std::move(plugin));
  }

  FlutterWebviewDesktopPlugin::FlutterWebviewDesktopPlugin(flutter::TextureRegistrar *textures,
                                                           flutter::BinaryMessenger *messenger)
      : textures_(textures), messenger_(messenger)
  {
    // window_class_.lpszClassName = L"FlutterWebviewDesktop";
    // window_class_.lpfnWndProc = WndProc;
    // // window_class_.hInstance = GetModuleHandle(L"flutter_webview_desktop_plugin");
    // RegisterClass(&window_class_);
  }

  FlutterWebviewDesktopPlugin::~FlutterWebviewDesktopPlugin()
  {
    instances_.clear();
    // WebviewCefShutDown();
    // UnregisterClass(window_class_.lpszClassName, nullptr);
    // if (window_handle_)
    // {
    //   DestroyWindow(window_handle_);
    //   window_handle_ = nullptr;
    // }
    // g_webview_client_ = nullptr;
    g_app_ = nullptr;
  }

  ///创建webview
  void FlutterWebviewDesktopPlugin::InitWebviewBrowser(std::unique_ptr<flutter::MethodResult<flutter::EncodableValue>> result, int width, int height, const std::string &url, double devicePixelRatio)
  {
    CEF_REQUIRE_UI_THREAD();
    is_init_ = true;
    is_init_cef_ = true;
    CefRefPtr<CefCommandLine>
        command_line =
            CefCommandLine::GetGlobalCommandLine();

    // Create the browser using the Views framework if "--use-views" is specified
    // via the command-line. Otherwise, create the browser using the native
    // platform framework.
    const bool use_views = command_line->HasSwitch("use-views");

    // CefRefPtr<WebviewClient> webview = new WebviewClient(use_views, width, height, devicePixelRatio);
    // BrowserHandler implements browser-level callbacks.
    // CefRefPtr<WebviewClient>
    //     handler(webview);
    // g_app_->UpdateDelegate(webview.get());
    // // Specify CEF browser settings here.

    // #if defined(OS_WIN)

    //     if (window_handle_ == nullptr)
    //     {
    //       window_handle_ = CreateWindowEx(0, window_class_.lpszClassName, L"", 0, WS_CHILD | WS_VISIBLE,
    //                                       CW_DEFAULT, 0, 0, HWND_MESSAGE, nullptr,
    //                                       window_class_.hInstance, nullptr);
    //       //使用
    //       window_info.SetAsWindowless(window_handle_);
    //       window_info.shared_texture_enabled = true;
    //     }

    // #endif

    // Create the first browser window.

    auto bridge = std::make_unique<WebviewBridge>(
        textures_, messenger_, use_views, width, height, devicePixelRatio, url);
    auto texture_id = bridge->texture_id();
    instances_[texture_id] = std::move(bridge);
    auto response = flutter::EncodableValue(flutter::EncodableMap{
        {flutter::EncodableValue("textureId"),
         flutter::EncodableValue(texture_id)},
    });


    result->Success(response);
  }

  void FlutterWebviewDesktopPlugin::HandleMethodCall(
      const flutter::MethodCall<flutter::EncodableValue> &method_call,
      std::unique_ptr<flutter::MethodResult<flutter::EncodableValue>> result)
  {
    if (method_call.method_name().compare(kMethodInitialize) == 0)
    {
      const auto &map = std::get<flutter::EncodableMap>(*method_call.arguments());
      const auto width = map.find(flutter::EncodableValue("width"));
      const auto height = map.find(flutter::EncodableValue("height"));
      const auto initialUrl = map.find(flutter::EncodableValue("initialUrl"));
      const auto devicePixelRatio = map.find(flutter::EncodableValue("devicePixelRatio"));

      const auto widthValue = std::get_if<int>(&width->second);
      const auto heightValue = std::get_if<int>(&height->second);
      const auto initialUrlValue = std::get_if<std::string>(&initialUrl->second);
      const auto devicePixelRatioValue = std::get_if<double>(&devicePixelRatio->second);
      InitWebviewBrowser(std::move(result), *widthValue, *heightValue, *initialUrlValue, *devicePixelRatioValue);
    }
    else if (method_call.method_name().compare(kMethodDispose) == 0)
    {
      // WebviewClient::GetInstance()->Dispose();
      if (const auto texture_id = std::get_if<int64_t>(method_call.arguments()))
      {
        const auto it = instances_.find(*texture_id);
        if (it != instances_.end())
        {
          instances_.erase(it);
          return result->Success();
        }
      }

      return result->Success();
    }
    // /// addJavascriptChannels
    else if (method_call.method_name().compare(kMethodAddJavascriptChannels) == 0)
    {

      const auto funcName = std::get_if<std::string>(method_call.arguments());
      g_app_->UpdateFunName(*funcName);
      return result->Success();
    }

    // /// evaluateJavascript
    //  if (method_name.compare(kMethodEvaluateJavascript))
    // {
    //     // return result->NotImplemented();
    // }
    else
    {
      result->NotImplemented();
    }
  }

} // namespace

void FlutterWebviewDesktopPluginRegisterWithRegistrar(
    FlutterDesktopPluginRegistrarRef registrar)
{
  FlutterWebviewDesktopPlugin::RegisterWithRegistrar(
      flutter::PluginRegistrarManager::GetInstance()
          ->GetRegistrar<flutter::PluginRegistrarWindows>(registrar));
}

///
void WebviewInitCEF(HINSTANCE hInstance)
{
  // Enable High-DPI support on Windows 7 or newer.
  CefEnableHighDPISupport();

#if defined(CEF_USE_SANDBOX)
  // Manage the life span of the sandbox information object. This is necessary
  // for sandbox support on Windows. See cef_sandbox_win.h for complete details.
  CefScopedSandboxInfo scoped_sandbox;
  sandbox_info = scoped_sandbox.sandbox_info();
#endif

  CefMainArgs main_args(hInstance);

  // Create a temporary CommandLine object.
  CefRefPtr<CefCommandLine> command_line = CreateCommandLine(main_args);
  g_app_ = new WebviewApp();
  // CEF applications have multiple sub-processes (render, plugin, GPU, etc)
  // that share the same executable. This function checks the command-line and,
  // if this is a sub-process, executes the appropriate logic.
  int exit_code = CefExecuteProcess(main_args, g_app_, NULL);
  if (exit_code >= 0)
  {
    // The sub-process has completed so return here.
    return;
  }
  // Specify CEF global settings here.
  CefSettings settings;
  settings.no_sandbox = true;
  settings.multi_threaded_message_loop = false;
  settings.windowless_rendering_enabled = true;

  // Initialize CEF for the browser process. The first browser instance will be
  // created in CefBrowserProcessHandler::OnContextInitialized() after CEF has
  // been initialized.
  CefInitialize(main_args, settings, g_app_.get(), NULL);
  
}
void WebviewCefRunloop()
{
  // Run the CEF message loop. This will block until CefQuitMessageLoop() is
  // called.
  if (is_init_cef_){
    CefDoMessageLoopWork();
  }
  
}

void WebviewCefShutDown()
{
  g_app_ = nullptr;
  is_init_cef_ = false;
  // Shut down CEF.
  CefShutdown();
}