#include "webview_client.h"
#include <sstream>
// #include <string>
#include <atlstr.h>
#include "include/base/cef_callback.h"
#include "include/cef_app.h"
// #include "include/cef_parser.h"
#include "include/views/cef_browser_view.h"
#include "include/views/cef_window.h"
#include "include/wrapper/cef_closure_task.h"
#include "include/wrapper/cef_helpers.h"
#include "util_win.h"
#include <iostream>
#define _CRT_SECURE_NO_WARNINGS

WebviewClient::WebviewClient(bool use_views, int width, int height, double devicePixelRatio)
    : use_views_(use_views), is_closing_(false)
{

  if (width != texture_size_.width || height != texture_size_.height || devicePixelRatio != texture_size_.devicePixelRatio)
  {
    texture_size_ = {width, height, devicePixelRatio};
  }
}

WebviewClient::~WebviewClient()
{

  // browser_->GetHost()->CloseBrowser(true);
  // browser_ = nullptr;

  // std::cout << "WebviewBridge::~WebviewClient" << std::endl;
}

void WebviewClient::OnTitleChange(CefRefPtr<CefBrowser> browser,
                                  const CefString &title)
{
  CEF_REQUIRE_UI_THREAD();

  if (use_views_)
  {
    // Set the title of the window using the Views framework.
    CefRefPtr<CefBrowserView> browser_view =
        CefBrowserView::GetForBrowser(browser);
    if (browser_view)
    {
      CefRefPtr<CefWindow> window = browser_view->GetWindow();
      if (window)
        window->SetTitle(title);
    }
  }
  else if (!IsChromeRuntimeEnabled())
  {
    // Set the title of the window using platform APIs.
    PlatformTitleChange(browser, title);
  }
}
bool WebviewClient::OnProcessMessageReceived(CefRefPtr<CefBrowser> browser,
                                             CefRefPtr<CefFrame> frame,
                                             CefProcessId source_process,
                                             CefRefPtr<CefProcessMessage> message)
{

  CEF_REQUIRE_UI_THREAD();
  CefString name = message->GetName();
  CefRefPtr<CefListValue> args = message->GetArgumentList();
  if (name == "postMessage")
  {
    CefString name = args->GetString(0);
    CefString str = args->GetString(1);
    // std::wstring strParams;
    // strParams += args->GetString(0).ToWString();
    // strParams += args->GetString(1).ToWString();
    // strParams += args->GetString(2).ToWString();

    // CString strJs;
    // strJs.Format(_T("alert('browser %s')"), strParams.c_str());
    // browser->GetMainFrame()->ExecuteJavaScript(strJs.GetBuffer(0), "", 0);

    js_handle_changed_callback_(name, str);
    return true;
  }

  return false;
}

// static
bool WebviewClient::IsChromeRuntimeEnabled()
{
  static int value = -1;
  if (value == -1)
  {
    CefRefPtr<CefCommandLine> command_line =
        CefCommandLine::GetGlobalCommandLine();
    value = command_line->HasSwitch("enable-chrome-runtime") ? 1 : 0;
  }
  return value == 1;
}

///
void WebviewClient::LoadUrl(const std::string &url)
{
  if (!browser_)
    return;

  browser_->GetMainFrame()->LoadURL(url);
}

void WebviewClient::LoadStringContent(const std::string &content)
{
  if (!browser_)
    return;
  CefRefPtr<CefRequest> request(CefRequest::Create());

  // Add post data to the request.  The correct method and content-
  // type headers will be set by CEF.
  CefRefPtr<CefPostDataElement> postDataElement(CefPostDataElement::Create());
  postDataElement->SetToBytes(content.length(), content.c_str());
  CefRefPtr<CefPostData> postData(CefPostData::Create());
  postData->AddElement(postDataElement);
  request->SetPostData(postData);

  // Add a custom header
  CefRequest::HeaderMap headerMap;
  headerMap.insert(std::make_pair("X-My-Header", "My Header Value"));
  request->SetHeaderMap(headerMap);

  // Load the request
  browser_->GetMainFrame()->LoadRequest(request);
}
void WebviewClient::GoBack()

{
  if (!browser_)
    return;
  browser_->GoBack();
}
void WebviewClient::GoForward()
{
  if (!browser_)
    return;
  browser_->GoForward();
}
void WebviewClient::Reload()
{
  if (!browser_)
    return;
  browser_->Reload();
}

void WebviewClient::StopLoad()
{
  if (!browser_)
    return;
  browser_->StopLoad();
}

/// page 位置
void WebviewClient::SetCursorOffset(int x, int y)
{
  if (!browser_)
    return;
  // GetBrowser()->GetHost
}
void WebviewClient::SetTextureSize(int width, int height, double devicePixelRatio)
{
  if (!browser_)
    return;
  texture_size_ = {width, height, devicePixelRatio};
  browser_->GetHost()->WasResized();
}
void WebviewClient::Dispose()
{
  if (!browser_)
    return;
  cursor_changed_callback_ = nullptr;
  load_state_callback_ = nullptr;
  history_changed_callback_ = nullptr;
  url_changed_callback_ = nullptr;
  if (sw_buffer_)
  {
    sw_buffer_.reset();
  }
  if (pixel_buffer_)
  {
    pixel_buffer_.release();
  }

  browser_->GetHost()
      ->CloseBrowser(true);
}
void WebviewClient::SetMouseClick(WebviewClientPointerButton button, bool isDown, int x, int y)
{
  if (!browser_)
    return;
  CefMouseEvent mouse;
  mouse.x = x;
  mouse.y = y;
  mouse.modifiers = 0;

  cef_mouse_button_type_t ctype;
  switch (button)
  {
  case WebviewClientPointerButton::Middle:
    ctype = MBT_MIDDLE;
    break;
  case WebviewClientPointerButton::Right:
    ctype = MBT_RIGHT;
    break;
  case WebviewClientPointerButton::Left:
    ctype = MBT_LEFT;
  default:
    ctype = MBT_LEFT;
    break;
  }
  CefWindowHandle hwnd = browser_->GetHost()->GetWindowHandle();
  if (hwnd)
  {
    ::SetFocus(hwnd);
  }

  browser_->GetHost()->SendMouseClickEvent(mouse, ctype, !isDown, 1);
}
void WebviewClient::SetMouseMove(int x, int y, int deltaX, int deltaY)
{
  if (!browser_)
    return;
  CefMouseEvent mouse_event;
  mouse_event.x = x;
  mouse_event.y = y;
  // ApplyPopupOffset(mouse_event.x, mouse_event.y);
  // DeviceToLogical(mouse_event, device_scale_factor_);
  // mouse_event.modifiers = 0;
  browser_->GetHost()->SendMouseMoveEvent(mouse_event, false);
}
void WebviewClient::OnKeyEvent(UINT message, WPARAM const wparam,
                               LPARAM const lparam)
{
  if (!browser_)
    return;

  CefKeyEvent event;
  event.windows_key_code = wparam;
  event.native_key_code = lparam;
  event.is_system_key = message == WM_SYSCHAR || message == WM_SYSKEYDOWN ||
                        message == WM_SYSKEYUP;

  if (message == WM_KEYDOWN || message == WM_SYSKEYDOWN)
    event.type = KEYEVENT_RAWKEYDOWN;
  else if (message == WM_KEYUP || message == WM_SYSKEYUP)
    event.type = KEYEVENT_KEYUP;
  else
    event.type = KEYEVENT_CHAR;
  event.modifiers = client::GetCefKeyboardModifiers(wparam, lparam);

  // mimic alt-gr check behaviour from
  // src/ui/events/win/events_win_utils.cc: GetModifiersFromKeyState
  if ((event.type == KEYEVENT_CHAR) && client::IsKeyDown(VK_RMENU))
  {
    // reverse AltGr detection taken from PlatformKeyMap::UsesAltGraph
    // instead of checking all combination for ctrl-alt, just check current char
    HKL current_layout = ::GetKeyboardLayout(0);

    // https://docs.microsoft.com/en-gb/windows/win32/api/winuser/nf-winuser-vkkeyscanexw
    // ... high-order byte contains the shift state,
    // which can be a combination of the following flag bits.
    // 1 Either SHIFT key is pressed.
    // 2 Either CTRL key is pressed.
    // 4 Either ALT key is pressed.
    SHORT scan_res = ::VkKeyScanExW(wparam, current_layout);
    constexpr auto ctrlAlt = (2 | 4);
    if (((scan_res >> 8) & ctrlAlt) == ctrlAlt)
    { // ctrl-alt pressed
      event.modifiers &= ~(EVENTFLAG_CONTROL_DOWN | EVENTFLAG_ALT_DOWN);
      event.modifiers |= EVENTFLAG_ALTGR_DOWN;
    }
  }

  browser_->GetHost()->SendKeyEvent(event);
}

void WebviewClient::SetMouseDoubleClick(WebviewClientPointerButton button, bool isDown, int x, int y)
{
  if (!browser_)
    return;
  CefMouseEvent mouse;
  mouse.x = x;
  mouse.y = y;
  mouse.modifiers = 0;

  cef_mouse_button_type_t ctype;
  switch (button)
  {
  case WebviewClientPointerButton::Middle:
    ctype = MBT_MIDDLE;
    break;
  case WebviewClientPointerButton::Right:
    ctype = MBT_RIGHT;
    break;
  case WebviewClientPointerButton::Left:
    ctype = MBT_LEFT;
  default:
    ctype = MBT_LEFT;
    break;
  }

  browser_->GetHost()->SendMouseClickEvent(mouse, ctype, !isDown, 2);
}

void WebviewClient::SetMouseWheel(int x, int y, int deltaX, int deltaY)
{
  if (!browser_)
    return;
  CefMouseEvent mouse;
  mouse.x = x;
  mouse.y = y;
  browser_->GetHost()->SendMouseWheelEvent(mouse, deltaX, deltaY);
}

void WebviewClient::SetWindowOpenPolicy(WebviewClientWindowOpenPolicy policy)
{
  popup_window_policy_ = policy;
}

void WebviewClient::EvaluateJavaScript(const std::string &url, const std::string &jsCode)
{
  if (!browser_)
    return;

  browser_->GetMainFrame()->ExecuteJavaScript(jsCode,
                                              browser_->GetMainFrame()->GetURL(), 0);
}

// void WebviewClient::CallCppHanlder(const std::string &name, const std::string &result)
// {
//   ::MessageBox(NULL, L"CallCppHanlder1111", L"Tip", MB_OK);
//   ::MessageBox(NULL, L"CallCppHanlder", L"Tip", MB_OK);
//   url_changed_callback_("ss");
// }
